﻿using ProjectAirplaneWithRadar.Generics;
using System.Collections.Generic;
using ProjectAirplaneWithRadar.DrawningObjects;
using ProjectAirplaneWithRadar.MovementStrategy;
namespace ProjectAirplaneWithRadar.Generics
{
    internal class AirplanesGenericCollection<T, U>
    where T : DrawningAirplane
    where U : IMoveableObject
    {
        private readonly int _pictureWidth;
        private readonly int _pictureHeight;
        private readonly int _placeSizeWidth = 215;
        private readonly int _placeSizeHeight = 90;
        private readonly SetGeneric<T> _collection;
        public AirplanesGenericCollection(int picWidth, int picHeight)
        {
            int width = picWidth / _placeSizeWidth;
            int height = picHeight / _placeSizeHeight;
            _pictureWidth = picWidth;
            _pictureHeight = picHeight;
            _collection = new SetGeneric<T>(width * height);
        }
        public static int operator +(AirplanesGenericCollection<T, U> collect, T? obj)
        {
            if (obj == null || collect == null)
            {
                return -1;
                
            }
            return collect._collection.Insert(obj);
        }
        public static T? operator -(AirplanesGenericCollection<T, U> collect, int pos)
        {
            T? obj = collect._collection.Get(pos);
            if (obj != null)
            {
                collect._collection.Remove(pos);
            }
            return obj;
        }
        public U? GetU(int pos)
        {
            return (U?)_collection.Get(pos)?.GetMoveableObject;
        }
        public Bitmap ShowAirplanes()
        {
            Bitmap bmp = new(_pictureWidth, _pictureHeight);
            Graphics gr = Graphics.FromImage(bmp);
            DrawBackground(gr);
            DrawObjects(gr);
            return bmp;
        }
        private void DrawBackground(Graphics g)
        {
            Pen pen = new(Color.Black, 3);
            for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++)
            {
                for (int j = 0; j < _pictureHeight / _placeSizeHeight +
                1; ++j)
                {
                    g.DrawLine(pen, i * _placeSizeWidth, j *
                    _placeSizeHeight, i * _placeSizeWidth + _placeSizeWidth / 2, j *
                    _placeSizeHeight);
                }
                g.DrawLine(pen, i * _placeSizeWidth, 0, i *
                _placeSizeWidth, _pictureHeight / _placeSizeHeight * _placeSizeHeight);
            }
        }
        private void DrawObjects(Graphics g)
        {
            for (int i = 0; i < _collection.Count; i++)
            {
                DrawningAirplane? airplane = _collection.Get(i);
                if (airplane != null)
                {
                    int inRow = _pictureWidth / _placeSizeWidth;
                    airplane.SetPosition(_pictureWidth - _placeSizeWidth - (i % inRow * _placeSizeWidth), i / inRow * _placeSizeHeight) ;
                    airplane.DrawTransport(g);
                }
            }
        }
    }
}