using ProjectAirplaneWithRadar.DrawningObjects;
using ProjectAirplaneWithRadar.MovementStrategy;

namespace ProjectAirplaneWithRadar
{
    public partial class FormAirplaneWithRadar : Form
    {
        private DrawningAirplane _drawningAirplane;
        private AbstractStrategy _abstractStrategy;
        public DrawningAirplane? SelectedAirplane { get; private set; }
        private void Draw()
        {
            if (_drawningAirplane == null)
            {
                return;
            }
            Bitmap bmp = new Bitmap(pictureBoxAirplaneWithRadar.Width, pictureBoxAirplaneWithRadar.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawningAirplane.DrawTransport(gr);
            pictureBoxAirplaneWithRadar.Image = bmp;
        }
        public FormAirplaneWithRadar()
        {
            InitializeComponent();
        }
        private void buttonMove_Click(object sender, EventArgs e)
        {
            if (_drawningAirplane == null)
            {
                return;
            }
            string name = ((Button)sender)?.Name ?? string.Empty;
            switch (name)
            {
                case "buttonUp":
                    _drawningAirplane.MoveTransport(Direction.Up);
                    break;
                case "buttonDown":
                    _drawningAirplane.MoveTransport(Direction.Down);
                    break;
                case "buttonLeft":
                    _drawningAirplane.MoveTransport(Direction.Left);
                    break;
                case "buttonRight":
                    _drawningAirplane.MoveTransport(Direction.Right);
                    break;
            }
            Draw();
        }
        private void buttonCreateAirplaneWithRadar_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color bodyColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            Color additionalColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
                bodyColor = dialog.Color;
            if (dialog.ShowDialog() == DialogResult.OK)
                additionalColor = dialog.Color;

            _drawningAirplane = new DrawningAirplaneWithRadar(random.Next(100, 300), random.Next(1000, 3000),
                bodyColor, additionalColor, true, true,
                pictureBoxAirplaneWithRadar.Width, pictureBoxAirplaneWithRadar.Height);
            Draw();
        }
        private void buttonCreateAirplane_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color color = Color.FromArgb(random.Next(0, 256),
            random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialog = new();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                color = dialog.Color;
            }
            _drawningAirplane = new DrawningAirplane(random.Next(100, 300),
            random.Next(1000, 3000), color,
            pictureBoxAirplaneWithRadar.Width, pictureBoxAirplaneWithRadar.Height);
            _drawningAirplane.SetPosition(random.Next(10, 100), random.Next(10,
            100));
            Draw();
        }
        private void buttonStep_Click(object sender, EventArgs e)
        {
            if (_drawningAirplane == null)
            {
                return;
            }
            if (comboBoxAirplane.Enabled)
            {
                switch (comboBoxAirplane.SelectedIndex)
                {
                    case 0:
                        _abstractStrategy = new MoveToCenter();
                        break;
                    case 1:
                        _abstractStrategy = new MoveToBorder();
                        break;

                    default:
                        break;
                };
                if (_abstractStrategy == null)
                {
                    return;
                }
                _abstractStrategy.SetData(new
                DrawningObjectAirplane(_drawningAirplane), pictureBoxAirplaneWithRadar.Width,
                pictureBoxAirplaneWithRadar.Height);
                comboBoxAirplane.Enabled = false;
            }
            if (_abstractStrategy == null)
            {
                return;
            }
            _abstractStrategy.MakeStep();
            Draw();
            if (_abstractStrategy.GetStatus() == Status.Finish)
            {
                comboBoxAirplane.Enabled = true;
                _abstractStrategy = null;
            }
        }
        private void buttonSelectAirplane_Click(object sender, EventArgs e)
        {
            SelectedAirplane = _drawningAirplane;
            DialogResult = DialogResult.OK;
        }
    }
}